/*

	// no argument --> current time
	var time_now = new TimeStamp()

	// 1 argument = interpret string as time with ':' separator
	var time_from_string = new TimeStamp('12:99:78')
	time_from_string.timeString()   // '13:40:18'

	// 2 args = hours, minutes
	var time_from_hh_mm = new TimeStamp(55,'67')
	time_from_hh_mm.timeString()    // '56:07:00'

	// 3 args = hours, minutes, seconds
	var time_from_hh_mm_ss = new TimeStamp(5,'45',78)
	time_from_hh_mm_ss.timeString() // '05:46:18'
	
	// undefined timestamp
	var time_undef = new TimeStamp(undefined)
	time_undef.timeString()         // '--:--:--'
	time_undef.isDefined()          // false
	time_from_string.isDefined()    // true

	// corrected timestamp
	time_redefined = new TimeStamp(-15,5,-1)
	time_redefined.timeString()     // '00:05:00'
	time_redefined.isDefined()      // true

	// various outputs
	var time1 = new TimeStamp(5,'45',78)
	time1.timeString()       // '05:46:18'
	time1.shortTimeString()  // '05:46'
	time1.durationString()   // '5h46'18"'
	time1.timerString()      // '5:46:18'
	time1.hours()            // 5
	time1.minutes()          // 46
	time1.seconds()          // 18
	time1.hoursString()      // '05'
	time1.minutesString()    // '46'
	time1.secondsString()    // '18'
	
	// various outputs
	var time2 = new TimeStamp(0,12,13)
	time2.timeString()       // '00:12:13'
	time2.durationString()   // '12'13"'
	time1.timerString()      // '12:13'

	// various outputs
	var time3 = new TimeStamp(undefined)
	time3.timeString()       // '--:--:--'
	time3.shortTimeString()  // '--:--'
	time3.durationString()   // '--'
	time1.timerString()      // '--:--'
	time3.hours()            // -1
	time3.minutes()          // -1
	time3.seconds()          // -1
	time3.hoursString()      // '--'
	time3.minutesString()    // '--'
	time3.secondsString()    // '--'

*/

function TimeStamp()
{
	// if ( !(this instanceof TimeStamp) )
	// 	return TimeStamp.apply(TimeStamp, arguments); 

	// no arguments ==> present time
	if ( arguments.length < 1 ) {
		var now = new Date()
		return new TimeStamp(now.getHours(), now.getMinutes(), now.getSeconds())
	}
	
	// one argument only = string
	if ( arguments.length == 1 ) {
		var string_value = arguments[0]
		if ( !string_value )
			string_value = "--:--:--"
		var components = string_value.split(':')
		if ( ( ! components ) || components.length != 3 )
			var components = [ '--' , '--', '--' ]
		return new TimeStamp(components[0], components[1], components[2])
	}
	
	// 2 arguments = hours and mins
	if ( arguments.length == 2 )
		return new TimeStamp(arguments[0], arguments[1], -1)

	// 3 or more arguments
	components = [ arguments[0], arguments[1], arguments[2] ]
	
	// initial values for hours
	if ( components[0] == '--' || Math.floor(components[0]) < 0 || (!components[0]) )
		this._hours = -1
	else
		this._hours = 0 + Math.floor(components[0])

	// initial values for minutes
	if ( components[1] == '--' || Math.floor(components[1]) < 0 || (!components[1]) )
		this._minutes = -1
	else
		this._minutes = 0 + Math.floor(components[1])

	// initial values for seconds
	if ( components[2] == '--' || Math.floor(components[2]) < 0 || (!components[2]) )
		this._seconds = -1
	else
		this._seconds = 0 + Math.floor(components[2])
	
	// all fields are undefined
	if ( this._hours < 0 && this._minutes < 0 && this._seconds < 0 ) {
		this._hoursString   = '--'
		this._minutesString = '--'
		this._secondsString = '--'
	}
	
	// at least one field is defined
	else {
		
		// all values should be defined and != -1
		if ( this._hours < 0 )
			this._hours = 0
		if ( this._minutes < 0 )
			this._minutes = 0
		if ( this._seconds < 0 )
			this._seconds = 0
		
		// adjust secs and mins
		while ( this._seconds > 59 ) {
			this._minutes ++
			this._seconds -= 60
		}

		// adjust mins and hrs
		while ( this._minutes > 59 ) {
			this._hours ++
			this._minutes -= 60
		}
		
		// adjust hours
		if ( this._hours > 99 )
			this._hours = 99

		// string value
		this._hoursString = ( this._hours < 10 ? '0':'') + this._hours
		this._minutesString = ( this._minutes < 10 ? '0':'') + this._minutes
		this._secondsString = ( this._seconds < 10 ? '0':'') + this._seconds

	}
	
	// time strings
	this._timeString = this._hoursString + ':' + this._minutesString + ':' + this._secondsString 
	this._shortTimeString = this._hoursString + ':' + this._minutesString
	
	// duration string
	this._durationString = this._secondsString + '"'
	if ( this._hours < 0 )
		this.durationString = '--'
	else if ( this._hours > 0 )
		this._durationString = "" + this._hours + "h" + this._minutesString
	else if ( this._minutes > 0 )
		this._durationString = "" + this._minutes + "' " + this._secondsString + '"'
	else
		this._durationString = "" + this._seconds + '"'
	
	// timer string
	this._timerString = '--:--'
	if ( this._hours < 0 )
		this._timerString = '--:--'
	else if ( this._hours > 0 )
		this._timerString = "" + this._hours + ":" + this._minutesString + ":" + this._secondsString
	else
		this._timerString = "" + this._minutes + ":" + this._secondsString
	
	// various outputs
	this.timeString      = function() { return this._timeString      } // '05:46:18'
	this.shortTimeString = function() { return this._shortTimeString } // '05:46'
	this.durationString  = function() { return this._durationString  } // '5:46'18"'
	this.timerString     = function() { return this._timerString     } // '5:46:18'
	this.hoursString     = function() { return this._hoursString     } // '05'
	this.minutesString   = function() { return this._minutesString   } // '46'
	this.secondsString   = function() { return this._secondsString   } // '18'
	this.hours           = function() { return this._hours           } // 5
	this.minutes         = function() { return this._minutes         } // 46
	this.seconds         = function() { return this._seconds         } // 18

	// isDefined
	this.isDefined = function() { return this._hours >= 0 }

	// TimeStamp algebra
	this.minusTimeStamp = function(that) {
		if ( this.hours < 0 || that.hours < 0 )
			return new TimeStamp(-1,-1,-1)
		h = this.hours() - that.hours()
		m = this.minutes() - that.minutes()
		s = this.seconds() - that.seconds()
		while ( s < 0 ) { s+=60 ; m-- }
		while ( m < 0 ) { m+=60 ; h-- }
		if ( h < 0 )
			return new TimeStamp(-1,-1,-1)
		return new TimeStamp(h,m,s)
	}

	this.plusTimeStamp = function(that) {
		if ( this.hours < 0 || that.hours < 0 )
			return new TimeStamp(-1,-1,-1)
		h = this.hours() + that.hours()
		m = this.minutes() + that.minutes()
		s = this.seconds() + that.seconds()
		return new TimeStamp(h,m,s)
	}
}

// tests

ts1 = new TimeStamp("10:25:1")
if ( ts1.hours() != 10 || ts1.minutes() != 25 || ts1.seconds() != 1 )
	argh();
if ( ts1.timeString() != '10:25:01' )
	argh();

ts2 = new TimeStamp('14:26:63')
if ( ts2.timeString() != '14:27:03' )
	argh();

ts3 = ts2.minusTimeStamp(ts1)
if ( ts3.timeString() != '04:02:02' )
	argh();

ts4 = ts2.plusTimeStamp(ts1)
if ( ts4.timeString() != '24:52:04' )
	argh();

